package models

import (
	"gogs.mikescher.com/BlackForestBytes/goext/rfctime"
)

type PostType string //@enum:type

const (
	PostTypeLink    PostType = "LINK_POST"
	PostTypeVideo   PostType = "VIDEO_POST"
	PostTypeGallery PostType = "GALLERY_POST"
	PostTypeText    PostType = "TEXT_POST"
	PostTypeFile    PostType = "FILE_POST"
)

type PostParentType string //@enum:type

const (
	PostParentGroup PostParentType = "GROUP"
	PostParentEvent PostParentType = "EVENT"
	PostParentRoot  PostParentType = "ROOT"
)

type Post struct {
	PostID           PostID                   `json:"id"               bson:"_id,omitempty"`
	FrontendID       *string                  `json:"frontendID"       bson:"frontendID"`
	SolseitID        *int                     `json:"solseitID"        bson:"solseitID"`
	FrontendEnabled  bool                     `json:"frontendEnabled"  bson:"frontendEnabled"`
	AutoEnableTime   *rfctime.RFC3339NanoTime `json:"autoEnableTime"   bson:"autoEnableTime"`
	Title            string                   `json:"title"            bson:"title"`
	Type             PostType                 `json:"type"             bson:"type"`
	Text             string                   `json:"text"             bson:"text"`
	Images           []BlobID                 `json:"images"           bson:"images"`
	Files            []BlobID                 `json:"files"            bson:"files"`
	URL              string                   `json:"url"              bson:"url"`
	ImageURL         string                   `json:"imageURL"         bson:"imageURL"`
	CreationTime     rfctime.RFC3339NanoTime  `json:"creationTime"     bson:"creationTime"`
	LastModifiedTime rfctime.RFC3339NanoTime  `json:"lastModifiedTime" bson:"lastModifiedTime"`
	CreatorRef       AnyID                    `json:"creatorRef"       bson:"creatorRef"`
	CreatorType      EntryCreatorType         `json:"creatorType"      bson:"creatorType"`
	OrganizationID   OrganizationID           `json:"organizationID"   bson:"organizationID"`
	Parent           PostParent               `json:"parent"           bson:"parent"`
	External         PostExternalData         `json:"external"         bson:"external"`
	Deleted          *rfctime.RFC3339NanoTime `json:"deleted"          bson:"deleted"`
}

type PostParent struct {
	Type        PostParentType `json:"type"      bson:"type"`
	ReferenceID AnyID          `json:"reference" bson:"reference"`
}

type PostExternalData struct {
	Solseit *SolseitArticle
	//DCX DCXArticle
}
